#ifndef DSPOUT_H_INCLUDED
#define DSPOUT_H_INCLUDED

#include <qstring.h>

class AudioBuffer;

/**
 * @short Base class for DSP audio devices.
 * @author Billy Biggs <vektor@div8.net>
 *
 * This is the base class for DSP output devices, such as
 * /dev/dsp for OSS, or /dev/audio for Sun audio.
 */ 
class DspOut
{
public:
	/**
	 * DspOut constructor just does simple initialization.
	 */
	DspOut();

	/**
	 * DspOut destructor.
	 */
	virtual ~DspOut();

	enum DeviceState {
		DeviceOpened,
		DeviceClosed };

	/**
	 * This is the call to actually open the device for
	 * audio output.  All setup (samplerate, format) must be
	 * done before the call to openDevice().
	 */
	virtual bool openDevice() = 0;

	/**
	 * Closes the device.
	 */
	virtual bool closeDevice() = 0;

	/**
	 * Return the state of the device, opened or closed.
	 */
	DeviceState getDeviceState() const { return devstate; }

	/**
	 * This provides the caller with the next audio buffer to
	 * fill.  A call to @ref #writeBuffer () will cause this
	 * to actually be queued by the soundcard.
	 *
	 * The format of the audio buffer is that of the internal
	 * program format.  The call to @ref #writeBuffer () will
	 * perform the appropriate conversion to the output format,
	 * if any is required.
	 */
	AudioBuffer *getBuffer() { return audio_buf; }

	/**
	 * Returns the total number of audio fragments allocated
	 * by the soundcard.  This multiplied by the fragment size
	 * reveals the size of the soundcard buffer.
	 */
	unsigned int getTotalFragments() const { return totalfrags; }

	/**
	 * Returns the number of currently writable fragments without
	 * blocking.
	 */
	virtual unsigned int writableFragments() = 0;

	/**
	 * Send the current audio buffer (available using @ref #getBuffer () )
	 * to the soundcard.  This applies any conversion necessary to go
	 * from the internal format to the format required by the device.
	 */
	virtual bool writeBuffer() = 0;

	/**
	 * Returns a string containing the last error message from the
	 * soundcard, suitable for display to the user.  Filled whenever
	 * something goes wrong.
	 */
	QString getLastError() const { return lasterror; }

	/**
	 * Sets the number of channels for the output.  Currently, going
	 * over two (stereo) would not be very useful.  Returns true
	 * if successful, false otherwise.
	 *
	 * This must be called before the device is opened.
	 */
	virtual bool setNumChannels( int newchans ) = 0;

	/**
	 * Returns the number of channels the output is set to.
	 */
	int getNumChannels() const { return channels; }

	/**
	 * Attempts to set the sample rate, and returns false if it fails.
	 *
	 * This must be called before the device is opened.
	 */
	virtual bool setSampleRate( int newrate ) = 0;

	/**
	 * Returns the current sample rate of the output device.
	 */
	int getSampleRate() const { return rate; }

	/**
	 * Returns the name of the current output device.  This is simply
	 * for the user to be able to identify the device, and does not
	 * need to correspond to anything real, such as the filename.
	 */
	QString getDeviceName() const { return devname; }

	/**
	 * Sets the current output device name.  This is simply for the user
	 * to be able to identify the device, and does not need to correspond
	 * to anything real, such as the filename.
	 */
	void setDeviceName( const QString &newDevName );

protected:
	DeviceState devstate;	// Current state
	QString lasterror;	// Informative text output of last error
	QString devname;	// Device Name, (not necessarily filename!)

	AudioBuffer *audio_buf;	// Buffer that the application fills
	int channels;		// Number of channels (stereo/mono/etc)
	int rate;		// Sampling rate (in fps)
	int totalfrags;		// Total Frags
};

#endif  // DSPOUT_H_INCLUDED
